/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.commons.util.debug.DebugLevel;
import fr.amap.commons.util.debug.Debugger;
import fr.amap.lidar.amapvox.visualization.util.View;
import java.awt.BorderLayout;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import javax.swing.Action;
import javax.swing.JCheckBox;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.ScrollPaneConstants;
import org.jdesktop.swingx.JXButton;
import org.jdesktop.swingx.JXCollapsiblePane;
import org.jdesktop.swingx.JXComboBox;
import org.jdesktop.swingx.JXLabel;
import org.jdesktop.swingx.JXPanel;
import org.jdesktop.swingx.JXTitledPanel;
import fr.amap.lidar.amapvox.visualization.controllers.OptionsController;
import fr.amap.lidar.amapvox.visualization.opengl.util.ColorBars;
import fr.amap.lidar.amapvox.visualization.opengl.util.Skyboxes;
import fr.amap.lidar.amapvox.visualization.util.AvailablesLocales;
import fr.amap.lidar.amapvox.visualization.util.ProjectProperties;
import java.awt.CardLayout;
import java.awt.Color;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.border.LineBorder;
import org.jdesktop.swingx.combobox.EnumComboBoxModel;
import org.jdesktop.swingx.painter.MattePainter;

/**
 * Options view of the project.
 */
public class OptionsView extends View {

    private final OptionsController controller;

    private JXCollapsiblePane collapsiblePane;
    private JXTitledPanel titledPanel;
    private JXPanel titledPanelContent;
    private JScrollPane titledPanelScrollPane;

    private JXPanel localePanel;
    private JXLabel localeLabel;
    private JXComboBox localeComboBox;

    private JXPanel backgroundPanel;
    private JXLabel backgroundLabel;
    private JXComboBox backgroundComboBox;

    private JXPanel reflectionPanel;
    private JCheckBox reflectionCheckbox;

    private JXPanel colorbarPanel;
    private JXLabel colorbarLabel;
    private JXComboBox colorbarComboBox;
    
    private JXPanel shapePanel;
    private JXLabel shapeLabel;
    private JXComboBox shapeComboBox;

    private JXPanel drawFastPanel;
    private JCheckBox drawFastCheckbox;

    private JXPanel voxelSizePanel;
    private JXLabel voxelSizeLabel;
    private JSlider voxelSizeSlider;
    
    private JXPanel gridPanel;
    private JCheckBox gridCheckbox;
    
    private JXPanel boundingboxPanel;
    private JCheckBox boundingboxCheckbox;

    private JXPanel axisLegendaryPanel;
    private JXLabel axisLegendaryLabel;

    private Runnable onChangeColorBarCallback;

    /**
     * Creates new form RenderingFrame
     * @throws java.lang.Exception
     */
    public OptionsView() throws Exception {
        super();
        this.controller = new OptionsController();
    }

    public final OptionsController getController() {
        return controller;
    }

    public final void updateInitialProperties() {
        controller.changeEnableReflection(reflectionCheckbox.isSelected());
        controller.changeVoxelSize(voxelSizeSlider.getValue());
        controller.setShape(shapeComboBox.getSelectedIndex());
        drawFastCheckbox.setEnabled(shapeComboBox.getSelectedIndex() != 0);
        reflectionCheckbox.setEnabled(shapeComboBox.getSelectedIndex() != 0);
        controller.setHideGrid(!gridCheckbox.isSelected());
        controller.setHideBoundingbox(!boundingboxCheckbox.isSelected());
    }

    public void setOnChangeColorBarCallback(Runnable onChangeColorBarCallback) {
        this.onChangeColorBarCallback = onChangeColorBarCallback;
    }

    @Override
    protected void initContainer() throws Exception {
        setLocationRelativeTo(getParent());
    }

    @Override
    protected void initComponents() throws Exception {
        /* Create the collapsible panel */
        collapsiblePane = new JXCollapsiblePane(JXCollapsiblePane.Direction.LEFT);
        collapsiblePane.setAnimated(false);
        collapsiblePane.setLayout(new BorderLayout());

        final Action toggleAction = collapsiblePane.getActionMap().
                get(JXCollapsiblePane.TOGGLE_ACTION);

        toggleAction.putValue(JXCollapsiblePane.COLLAPSE_ICON,
                new javax.swing.ImageIcon(getClass().getResource("/icons/right.png")));
        toggleAction.putValue(JXCollapsiblePane.EXPAND_ICON,
                new javax.swing.ImageIcon(getClass().getResource("/icons/left.png")));

        final JXButton toggle = new JXButton(toggleAction);
        toggle.setText(null);
        toggle.setPreferredSize(new Dimension(20, 20));

        add(collapsiblePane, BorderLayout.CENTER);
        add(toggle, BorderLayout.WEST);

        /* Create title panel */
        titledPanel = new JXTitledPanel();
        titledPanel.getContentContainer().setLayout(new BorderLayout());
        addLocaleCallback((locale) -> {
            titledPanel.setTitle(bundle.getString("optionsView_Title"));
        });
        collapsiblePane.getContentPane().add(titledPanel);

        /* Create scroll pane */
        titledPanelContent = new JXPanel();
        titledPanelContent.setLayout(new BoxLayout(titledPanelContent, BoxLayout.PAGE_AXIS));

        titledPanelScrollPane = new JScrollPane();
        titledPanelScrollPane.setHorizontalScrollBarPolicy(
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        titledPanelScrollPane.setViewportView(titledPanelContent);
        titledPanel.getContentContainer().add(titledPanelScrollPane, BorderLayout.CENTER);

        /* Create the locale components */
        localePanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        localePanel.setMaximumSize(new Dimension(10000, 35));
        titledPanelContent.add(localePanel);

        localeLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            localeLabel.setText(bundle.getString("optionsView_localeLabel") + " :");
        });
        localePanel.add(localeLabel);

        localeComboBox = new JXComboBox();
        final ComboBoxModel model = new EnumComboBoxModel(AvailablesLocales.class);
        localeComboBox.setModel(model);
        addLocaleCallback((locale) -> {
            localeComboBox.setSelectedItem(AvailablesLocales.getFromLocale(locale));
            localeComboBox.setToolTipText(bundle.getString("optionsView_localeCombobox_tooltip"));
        });
        localeComboBox.addActionListener((java.awt.event.ActionEvent evt) -> {
            if (parentView != null) {
                final AvailablesLocales locale = ((AvailablesLocales) localeComboBox.getSelectedItem());
                parentView.setLocale(locale.getLocale());
                ProjectProperties.setProperty("language", locale.name());
            }
        });
        localePanel.add(localeComboBox);

        /* Create colorbar components */
        colorbarPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        colorbarPanel.setMaximumSize(new Dimension(10000, 35));

        colorbarLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            colorbarLabel.setText(bundle.getString("optionsView_colorbarLabel") + " :");
        });

        colorbarComboBox = new JXComboBox(ColorBars.getColorBarsNames());
        colorbarComboBox.setSelectedItem(ProjectProperties.getProperty("colorBar", "Equatorial"));
        addLocaleCallback((locale) -> {
            colorbarComboBox.setToolTipText(bundle.getString("optionsView_colorbarComboBox_tooltip"));
        });
        colorbarComboBox.addActionListener((e) -> {
            /* Change the color bar of the voxel space */
            controller.changeColorBar((String) colorbarComboBox.getSelectedItem());

            if (onChangeColorBarCallback != null) {
                onChangeColorBarCallback.run();
            }
        });

        colorbarPanel.add(colorbarLabel);
        colorbarPanel.add(colorbarComboBox);
        titledPanelContent.add(colorbarPanel);
        
        /* Create shape components */
        shapePanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        shapePanel.setMaximumSize(new Dimension(10000, 35));
        titledPanelContent.add(shapePanel);
        
        shapeLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            shapeLabel.setText(bundle.getString("optionsView_shapeLabel") + " :");
        });
        shapePanel.add(shapeLabel);
        
        shapeComboBox = new JXComboBox();
        addLocaleCallback((locale) -> {
            final ComboBoxModel shapeModel = new DefaultComboBoxModel(new String[] {
                bundle.getString("optionsView_shapeComboBox_pixel"),
                bundle.getString("optionsView_shapeComboBox_cube"),
            });
            shapeComboBox.setModel(shapeModel);
            shapeComboBox.setSelectedIndex(Integer.valueOf(ProjectProperties.getProperty("shape", "1")));
            shapeComboBox.setToolTipText(bundle.getString("optionsView_shapeComboBox_tooltip"));
        });
        shapeComboBox.addItemListener((e) -> {
            controller.setShape(shapeComboBox.getSelectedIndex());
            drawFastCheckbox.setEnabled(shapeComboBox.getSelectedIndex() != 0);
            reflectionCheckbox.setEnabled(shapeComboBox.getSelectedIndex() != 0);
        });
        shapePanel.add(shapeComboBox);

        /* Create voxel size components */
        voxelSizePanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        voxelSizePanel.setMaximumSize(new Dimension(10000, 35));

        voxelSizeLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            voxelSizeLabel.setText(bundle.getString("optionsView_voxelSizeSlider") + " :");
        });

        voxelSizeSlider = new JSlider(1, 100);
        voxelSizeSlider.setPreferredSize(new Dimension(125, 20));
        voxelSizeSlider.setValue(Integer.valueOf(
                ProjectProperties.getProperty("voxelSize", "100")));
        addLocaleCallback((locale) -> {
            voxelSizeSlider.setToolTipText(bundle.getString("optionsView_voxelSizeSlider_tooltip"));
        });
        voxelSizeSlider.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseReleased(MouseEvent e) {
                controller.changeVoxelSize(voxelSizeSlider.getValue());
            }
        });

        voxelSizePanel.add(voxelSizeLabel);
        voxelSizePanel.add(voxelSizeSlider);
        titledPanelContent.add(voxelSizePanel);
        
        /* Create draw fast components */
        drawFastPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        drawFastPanel.setMaximumSize(new Dimension(10000, 35));

        drawFastCheckbox = new JCheckBox();
        drawFastCheckbox.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        drawFastCheckbox.setSelected(Boolean.valueOf(
                ProjectProperties.getProperty("drawFastEnabled", "true")));
        addLocaleCallback((locale) -> {
            drawFastCheckbox.setText(bundle.getString("optionsView_drawFastCheckbox"));
            drawFastCheckbox.setToolTipText(bundle.getString("optionsView_drawFastCheckbox_tooltip"));
        });
        drawFastCheckbox.addActionListener((e) -> {
            controller.changeDrawFast(drawFastCheckbox.isSelected());
        });

        drawFastPanel.add(drawFastCheckbox);
        titledPanelContent.add(drawFastPanel);
        
        /* Create background components */
        backgroundPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        backgroundPanel.setMaximumSize(new Dimension(10000, 35));

        backgroundLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            backgroundLabel.setText(bundle.getString("optionsView_backgroundLabel") + " :");
        });

        backgroundComboBox = new JXComboBox(Skyboxes.getSkyNames());
        backgroundComboBox.setSelectedItem(ProjectProperties.getProperty("background", "Simple"));
        addLocaleCallback((locale) -> {
            backgroundComboBox.setToolTipText(bundle.getString("optionsView_backgroundComboBox_tooltip"));
        });
        backgroundComboBox.addActionListener((java.awt.event.ActionEvent evt) -> {
            try {
                /* Change the background of the scene */
                controller.changeBackground((String) backgroundComboBox.getSelectedItem());
            } catch (IOException ex) {
                Debugger.debugInterface(ex, DebugLevel.ERROR, parentView);
            }
        });

        backgroundPanel.add(backgroundLabel);
        backgroundPanel.add(backgroundComboBox);
        titledPanelContent.add(backgroundPanel);

        /* Create reflection components */
        reflectionPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        reflectionPanel.setMaximumSize(new Dimension(10000, 35));
        titledPanelContent.add(reflectionPanel);

        reflectionCheckbox = new JCheckBox();
        reflectionCheckbox.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        addLocaleCallback((locale) -> {
            reflectionCheckbox.setText(bundle.getString("optionsView_reflectionCheckbox"));
            reflectionCheckbox.setToolTipText(bundle.getString("optionsView_reflectionCheckbox_tooltip"));
        });
        reflectionCheckbox.addActionListener((e) -> {
            controller.changeEnableReflection(reflectionCheckbox.isSelected());
        });
        reflectionCheckbox.setSelected(Boolean.valueOf(
                ProjectProperties.getProperty("reflectionsEnabled", "true")));
        reflectionPanel.add(reflectionCheckbox);
        
        /* Create grid panel */
        gridPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        gridPanel.setMaximumSize(new Dimension(10000, 30));
        titledPanelContent.add(gridPanel);
        
        gridCheckbox = new JCheckBox();
        gridCheckbox.setSelected(!Boolean.valueOf(
                ProjectProperties.getProperty("hideGrid", "false")));
        gridCheckbox.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        addLocaleCallback((locale) -> {
            gridCheckbox.setText(bundle.getString("optionsView_gridCheckbox"));
            gridCheckbox.setToolTipText(bundle.getString("optionsView_gridCheckbox_tooltip"));
        });
        gridCheckbox.addActionListener((e) -> {
            controller.setHideGrid(!gridCheckbox.isSelected());
        });
        gridPanel.add(gridCheckbox);
        
        /* Create boundingbox panel */
        boundingboxPanel = new JXPanel(new FlowLayout(FlowLayout.LEFT));
        boundingboxPanel.setMaximumSize(new Dimension(10000, 30));
        titledPanelContent.add(boundingboxPanel);
        
        boundingboxCheckbox = new JCheckBox();
        boundingboxCheckbox.setSelected(!Boolean.valueOf(
                ProjectProperties.getProperty("hideBoundingbox", "false")));
        boundingboxCheckbox.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        addLocaleCallback((locale) -> {
            boundingboxCheckbox.setText(bundle.getString("optionsView_boundingboxCheckbox"));
            boundingboxCheckbox.setToolTipText(bundle.getString("optionsView_boundingboxCheckbox_tooltip"));
        });
        boundingboxCheckbox.addActionListener((e) -> {
            controller.setHideBoundingbox(!boundingboxCheckbox.isSelected());
        });
        boundingboxPanel.add(boundingboxCheckbox);
        
        /* Create axis legendary label */
        axisLegendaryPanel = new JXPanel(new CardLayout(20, 20));
        axisLegendaryPanel.setMaximumSize(new Dimension(10000, 190));
        titledPanelContent.add(axisLegendaryPanel);

        axisLegendaryLabel = new JXLabel(loadIcon("/icons/axisLegendary.png"));
        axisLegendaryLabel.setBackgroundPainter(new MattePainter(
                new Color((float) 208 / 255, (float) 214 / 255, (float) 217 / 255)));
        axisLegendaryLabel.setBorder(new LineBorder(Color.BLACK));
        axisLegendaryPanel.add(axisLegendaryLabel);

        pack();
    }

}
